import { ClientGrpc, ClientProxyFactory, Transport } from '@nestjs/microservices';
import { lastValueFrom, Observable } from 'rxjs';
import { join } from 'path';
import { Injectable } from '@nestjs/common';

interface GrpcResponse<T> {
  code: number;
  msg: string;
  data: T;
}

export function createGrpcService<T>(options: {
  serviceName: string;
  package: string;
  protoPath: string;
  url: string;
}) {
  @Injectable()
  class GrpcServiceBase {
    public client: ClientGrpc;
    public grpcService: T;

    constructor() {
      this.client = ClientProxyFactory.create({
        transport: Transport.GRPC,
        options: {
          url: options.url,
          package: options.package,
          protoPath: options.protoPath,
        },
      }) as ClientGrpc;
    }

    onModuleInit() {
      this.grpcService = this.client.getService<T>(options.serviceName);
    }

    async execute<Input, Output>(methodName: string, input: Input): Promise<Output> {
      const service = this.grpcService as any;
      if (!service[methodName]) {
        throw new Error(`Method ${methodName} not found in GRPC service`);
      }

      try {
        const observable = service[methodName](input);
        if (!observable) {
          throw new Error(`GRPC call returned null or undefined`);
        }

        const response = await lastValueFrom(observable as Observable<GrpcResponse<Output>>);
        
        // 添加调试日志
        console.log('GRPC Response:', response);
        
        // 检查响应是否存在
        if (!response) {
          throw new Error('GRPC response is undefined');
        }

        // 处理 Go 的 int32 类型
        const code = typeof response.code === 'number' ? response.code : 
                    (response as any).Code || (response as any).code || 0;
                    
        if (code !== 0) {
          const msg = response.msg || (response as any).Msg || (response as any).message || 'Unknown error';
          throw new Error(msg);
        }
        
        // 处理数据字段，考虑大小写
        const data = response.data || (response as any).Data;
        return data;
      } catch (error) {
        console.error('GRPC call error:', error);
        throw error;
      }
    }
  }

  return GrpcServiceBase;
} 