import {
  ArgumentsHost,
  Catch,
  HttpStatus,
  Inject,
  UnauthorizedException,
} from '@nestjs/common';
import { BaseExceptionFilter } from '@nestjs/core';
import { ApiError } from '../lib/apiError';
import { LoggerService } from '@common/logger/logger.service';
import { QyWechatbotService } from '@dt/nestjs-qywechatbot';

@Catch()
export class AllExceptionsFilter extends BaseExceptionFilter {
  constructor(
    @Inject(LoggerService) private readonly logger: LoggerService,
    @Inject(QyWechatbotService)
    private readonly qyWechatbot: QyWechatbotService,
  ) {
    super();
  }

  // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
  catch(exception: any, host: ArgumentsHost) {
    const ctx = host.switchToHttp();
    const request = ctx.getRequest();
    const response = ctx.getResponse();
    let status = HttpStatus.OK;
    let code = exception.getStatus
      ? exception.getStatus()
      : HttpStatus.INTERNAL_SERVER_ERROR;
    if (
      exception.isAxiosError &&
      exception.response &&
      exception.response.status
    ) {
      code = exception.response.status;
    } else if (exception instanceof ApiError) {
      code = exception.code;
    } else if (exception instanceof UnauthorizedException) {
      status = HttpStatus.UNAUTHORIZED;
    }

    let message = '';
    if (exception.response && exception.response.message) {
      if (typeof exception.response.message === 'string') {
        message = exception.response.message;
      } else {
        message = JSON.stringify(exception.response.message);
      }
    } else {
      message =
        typeof exception.message === 'string'
          ? exception.message
          : exception.message?.error;
    }
    const url = request?.url || request?.originalUrl || '';

    const error: { code: any; msg: any; stack: any; data?: any } = {
      code,
      msg: message,
      stack: exception.stack?.split('\n').slice(0, 2),
    };
    if (process.env.NODE_ENV !== 'container') {
      console.log('error =>>>', exception);
    }

    this.logger.error({
      ...error,
    });
    this.qyWechatbot.sendMarkdownMessage(
      `** HQ Http 错误报告**
** 消息**: ${error?.msg}
** 路径**: ${url}
** 错误码**: ${error?.code}
** Trace Id**: ${this.logger.dtlog.get('trace_id')}
** 环境**: ${process.env?.RABBIT_VHOST}
      `,
    );
    // 接口不返回堆栈信息
    const errResponse: { code: any; msg: any; result?: any } = {
      code: error.code,
      msg: error.msg,
    };
    if (exception instanceof ApiError) {
      if (exception.result) {
        errResponse.result = exception.result;
      }
      return response.status(status).send(errResponse);
    }
    console.log('url =>> ', url);

    response?.status &&
      typeof response?.status === 'function' &&
      response.status(status).send(errResponse);
  }
}
